package de.uniulm.mathematik.typo.fonts;

/**
 * Interface for simple font metrics whereas there is just one
 * writing direction from left to right, i.e. WY is zero for all
 * glyphs. This interface follows mainly the conventions of
 * Adobe Type 1 fonts:
 * <UL>
 * <LI>Code values range from 0 to 255.
 * <LI>The units are given in 1/1000 of the point size.
 * </UL>
 */
public interface FontMetrics {
   /**
    * Return the name of the font, e.g. "Times-Roman".
    * @return the name of the font as string.
    */
   public String getName();

   /**
    * Return the name of a character.
    * @return the name of a character as string, if any.
    */
   public String getName(int code);

   /**
    * Return the code of a character by name.
    * @return 0 is returned
    * when the name is not known, -1 if it has no assigned
    * code in the default encoding.
    */
   public int getCodeByName(String name);

   /**
    * Return lowest code number.
    */
   public int minCode();

   /**
    * Return highest code number.
    */
   public int maxCode();

   /**
    * Return number of characters within this font.
    */
   public int length();

   /**
    * Return true, if there is a character with the given code.
    */
   public boolean defined(int code);

   /**
    * Return font bounding box as array of four integers:
    * llx, lly, urx, and ury.
    */
   public int[] getBoundingBox();

   /**
    * Return bounding box of an individual character as array of
    * four integers: llx, lly, urx, and ury. Zeroes are returned
    * if there is no character with the given code or no bbox
    * has been provided for this character.
    */
   public int[] getBoundingBox(int code);

   /**
    * Return bounding box of an individual character as array of
    * four integers: llx, lly, urx, and ury. Zeroes are returned
    * if there is no character with the given code or no bbox
    * has been provided for this character.
    */
   public int[] getBoundingBox(String name);

   /**
    * Return the versal height of the font, e.g. the height of
    * versal letters.
    */
   public int getVersalHeight();

   /**
    * Return the height of the lower case letter x.
    */
   public int getXHeight();

   /**
    * Return the height between the baseline and the bottom of the
    * lower case letter p.
    */
   public int getDescender();

   /**
    * Return the height between the baseline and the height of the
    * lower case letter d.
    */
   public int getAscender();

   /**
    * Return width of an individual character. Zero is returned
    * if there is no character with the given code or no width
    * has been specified for this character.
    */
   public int getWidth(int code);

   /**
    * Return width of an individual character. Zero is returned
    * if there is no character with the given code or no width
    * has been specified for this character.
    */
   public int getWidth(String name);

   /**
    * Return the kerning for the given two characters. 0 is
    * returned if no kerning has been defined.
    */
   public int getKerning(int code1, int code2);

   /**
    * Return the kerning for the given two characters. 0 is
    * returned if no kerning has been defined.
    */
   public int getKerning(String name1, String name2);
}
