package de.uniulm.mathematik.typo.hyphen;

import de.uniulm.mathematik.tries.*;

/**
 * An instance of the simple hyphenator combines a hyphenation table with two
 * configuration parameters (lefthyphenmin and righthyphenmin) and
 * allows to hyphenate words that are sequences of letters. The
 * configuration parameter lefthyphenmin specifies the minimal length of
 * a word prefix that can be separated from a word through hyphenation.
 * The righthyphenmin parameter specifies the minimal length for a
 * suffix. Different weights are not supported, i.e. all weights equal 1.
 */
public class SimpleHyphenator implements Hyphenator {
   private int leftmin = 2;
   private int rightmin = 2;
   private TrieReader<HyphenationEntry> patterns;

   /**
    * Construct a hyphenator with leftmin and rightmin both set to 2.
    */
   public SimpleHyphenator(TrieReader<HyphenationEntry> trie) {
      patterns = trie;
      leftmin = 2; rightmin = 2;
   }

   /**
    * Construct a hyphenator with given configuration parameters.
    * @param lefthyphenmin The minimal length of a word prefix that
    *                      can be cut off from a word through hyphenation.
    * @param righthyphenmin The minimal length of a word suffix that
    *                       can be cut off from a word through hyphenation.
    */
   public SimpleHyphenator(TrieReader<HyphenationEntry> trie,
	 int lefthyphenmin, int righthyphenmin) {
      patterns = trie;
      assert lefthyphenmin >= 1 && righthyphenmin >= 1;
      leftmin = lefthyphenmin;
      rightmin = righthyphenmin;
   }

   /**
    * Return the minimal length of a word prefix that can be
    * cut off from a word through hyphenation.
    */
   public int getLeftMin() {
      return leftmin;
   }

   /**
    * Return the minimal length of a word suffix that
    * can be cut off from a word through hyphenation.
    */
   public int getRightMin() {
      return rightmin;
   }

   /**
    * Return minimal weight within the associated rules.
    */
   public int getMinWeight() {
      return 1;
   }

   /**
    * Return maximal weight within the associated rules.
    */
   public int getMaxWeight() {
      return 1;
   }

   /**
    * Return possible hyphenation positions for a given string.
    * @param s Sequence of letters that constitute a to be hyphenated
    *          word.
    * @return  Integer array with hyphenation positions in ascending
    *          order. If <I>s</I> is of length <I>n</I> then
    *          the hyphenation positions range from <I>lefthyphenmin</I>
    *          to <I>n-righthyphenmin</I>.
    */
   public int[] hyphenate(String s) {
      // extract array of codepoints
      // where upper case letters are mapped to lower case
      String word = "." + s + ".";
      int len = word.codePointCount(0, word.length());
      int[] codepoints = new int[len];
      for (int i = 0; i < word.length(); i = word.offsetByCodePoints(i, 1)) {
	 codepoints[i] = word.codePointAt(i);
	 if (Character.isUpperCase(codepoints[i])) {
	    codepoints[i] = Character.toLowerCase(codepoints[i]);
	 }
      }
      // find matching hyphenation patterns and apply them to value[]
      int[] value = new int[len+1];
      for (int start = 0; start+rightmin < len; ++start) {
	 int pos = start;
	 for (TrieReader.TriePointer<HyphenationEntry> ptr = patterns.getRoot();
	       ptr != null;
	       ptr = pos < len? ptr.descend(codepoints[pos++]): null) {
	    HyphenationEntry rule = ptr.getInfo();
	    if (rule != null) {
	       for (HyphenationPoint point: rule) {
		  int index = point.getPosition() + start - 1;
		  if (index >= leftmin && index + rightmin < len-1) {
		     int val = point.getValue();
		     if (val > value[index]) {
			value[index] = val;
		     }
		  }
	       }
	    }
	 }
      }
      // create array with hyphenation positions
      int count = 0;
      for (int i = 0; i <= len; ++i) {
	 if (value[i] % 2 == 1) {
	    ++count;
	 }
      }
      if (count == 0) {
	 return null;
      }
      int[] result = new int[count]; int index = 0;
      for (int i = 0; i <= len; ++i) {
	 if (value[i] % 2 == 1) {
	    result[index++] = i;
	 }
      }
      return result;
   }
}
