package de.uniulm.mathematik.typo.hyphen;

import java.util.*;

/**
 * Weighted hyphenation entries represent a possibly shared
 * sequence of weighted hyphenation points. They are hashable
 * such that common suffixes of weighted hyphenation point sequences
 * can be found using a hash table.
 */
public class WeightedHyphenationEntry
      implements WeightedHyphenationRule, WeightedHyphenationPoint {

   protected int pos;
   protected int weight;
   protected WeightedHyphenationEntry next;

   /**
    * This iterators allows to iterate through the weighted hyphenation points
    * beginning at this hyphenation entry.
    */
   public class WeightedHyphenationEntryIterator
	 implements Iterator<WeightedHyphenationPoint> {

      private WeightedHyphenationEntry entry;

      /**
       * Create iterator beginning at e.
       */
      public WeightedHyphenationEntryIterator(WeightedHyphenationEntry e) {
	 entry = e;
      }

      /**
       * Return true if there is at least one value left
       * which can be returned by the iterator.
       */
      public boolean hasNext() {
	 return entry != null;
      }

      /**
       * Not supported, raises UnsupportedOperationException.
       */
      public void remove() {
	 throw new UnsupportedOperationException();
      }

      /**
       * Return next hyphenation entry.
       * This method raises NoSuchElementException if hasNext()
       * would have returned false.
       */
      public WeightedHyphenationEntry next() {
	 if (entry == null) {
	    throw new NoSuchElementException();
	 }
	 WeightedHyphenationEntry e = entry;
	 entry = entry.next;
	 return e;
      }
   }

   /**
    * Return the first hyphenation position.
    */
   public int getPosition() {
      return pos;
   }

   /**
    * Return the first hyphenation weight. The larger the weight
    * the more favorable is a hyphenation at that point.
    */
   public int getWeight() {
      return weight;
   }

   /**
    * Return iterator that allows to examine all weighted hyphenation points
    * of this hyphenation entry.
    */
   public Iterator<WeightedHyphenationPoint> iterator() {
      return new WeightedHyphenationEntryIterator(this);
   }

   /**
    * Returns true iff object is an WeightedHyphenationEntry with an
    * equivalent sequence of hyphenation positions and values.
    */
   public boolean equals(Object object) {
      if (object instanceof WeightedHyphenationEntry) {
	 WeightedHyphenationEntry entry1 = this;
	 WeightedHyphenationEntry entry2 = (WeightedHyphenationEntry) object;
	 if (entry1 == entry2) return true;
	 for(;;) {
	    if (entry1.pos != entry2.pos || entry1.weight != entry2.weight) {
	       return false;
	    }
	    if ((entry1.next == null) != (entry2.next == null)) {
	       return false;
	    }
	    if (entry1.next == null) return true;
	    entry1 = entry1.next; entry2 = entry2.next;
	 }
      } else {
	 return false;
      }
   }

   /**
    * Returns a hash code that depends on the sequence
    * of hyphenation positions and values.
    */
   public int hashCode() {
      int hashval = 0;
      for (WeightedHyphenationEntry entry = this;
	    entry != null; entry = entry.next) {
	 hashval = Integer.rotateLeft(hashval, 3)
	    ^ (entry.pos << 4) ^ entry.weight;
      }
      return hashval;
   }

   /**
    * Create a new hyphenation entry with the first hyphenation point
    * defined by p and v. The remaining hyphenation points are given
    * by n, if non-null.
    */
   protected WeightedHyphenationEntry(int p, int w,
	 WeightedHyphenationEntry n) {
      assert n == null || p < n.pos;
      pos = p; weight = w; next = n;
   }

   /**
    * Returns a string that contains the comma-separated
    * sequence of hyphenation positions and their weights,
    * enclosed in braces. This is useful for debugging purposes.
    */
   public String toString() {
      StringBuffer result = new StringBuffer();
      result.append("{");
      WeightedHyphenationEntry entry = this;
      boolean first = true;
      while (entry != null) {
	 if (first) {
	    first = false;
	 } else {
	    result.append(", ");
	 }
	 result.append(entry.pos + ":" + entry.weight);
	 entry = entry.next;
      }
      result.append("}");
      return result.toString();
   }
}
