package de.uniulm.mathematik.typo.items;

import de.uniulm.mathematik.typo.fonts.*;

/**
 * The NormalizedGlyphBox class supports boxes for individual glyphs,
 * e.g. letters and other symbols. It works like GlyphBox but its
 * depth and height depend not on the individual metrics of the glyph
 * but that of the chosen font.
 */
public class NormalizedGlyphBox extends Box {
   private FontMetrics fm;
   private int encoding;
   private int codepoint;
   private int size;
   private int height;
   private int depth;
   private int width;

   /**
    * Construct a glyph box:
    * @param fm The metrics of the selected font.
    * @param size The font size in points (not units!).
    * @param encoding The encoding of the glyph.
    * @param codepoint The Unicode codepoint that corresponds to the glyph.
    */
   public NormalizedGlyphBox(FontMetrics fm, int size,
	 int encoding, int codepoint) {
      this.fm = fm;
      this.size = size;
      this.encoding = encoding;
      this.codepoint = codepoint;
      height = fm.getAscender() * size;
      if (height < 0) {
	 height = 0;
      }
      depth = -fm.getDescender() * size;
      if (depth < 0) {
	 depth = 0;
      }
      width = fm.getWidth(encoding) * size;
   }

   /**
    * Construct a glyph box:
    * @param fm The metrics of the selected font.
    * @param size The font size in points (not units!).
    * @param codepoint The Unicode codepoint of the glyph which is
    *        identical to its encoding. (This can be guaranteed for
    *        printable ASCII characters only.)
    */
   public NormalizedGlyphBox(FontMetrics fm, int size, int codepoint) {
      this.fm = fm;
      this.size = size;
      this.encoding = codepoint;
      this.codepoint = codepoint;
      height = fm.getAscender() * size;
      if (height < 0) {
	 height = 0;
      }
      depth = -fm.getDescender() * size;
      if (depth < 0) {
	 depth = 0;
      }
      width = fm.getWidth(encoding) * size;
   }

   /**
    * Returns the width of the glyph in units (1/1000 points).
    * This value is derived from the width returned by the
    * associated font metrics. This is independent from the
    * bounding box.
    */
   public int getWidth() {
      return width;
   }

   /**
    * Return the height of the glyph in units (1/1000 points).
    * This value is derived from the ascender of the corresponding
    * font.
    */
   public int getHeight() {
      return height;
   }

   /**
    * Return the depth of the glyph in units (1/1000 points).
    * This value is derived from the descender of the corresponding
    * font.
    */
   public int getDepth() {
      return depth;
   }

   /**
    * Return the character as Unicode codepoint.
    */
   public int getChar() {
      return codepoint;
   }

   /**
    * Return its font
    */
   public FontMetrics getFont() {
      return fm;
   }

   /**
    * Return its font size
    */
   public int getFontSize() {
      return size;
   }

   /**
    * Generate PostScript text that generates the glyph.
    * @param context The context is used to combine multiple
    *                subsequent characters to PostScript strings
    *                as far as possible.
    */
   public StringBuffer genPostScript(PostScriptContext context) {
      return context.addTo(fm, size, encoding);
   }

   /**
    * Return a printable string representing the internal state.
    */
   public String toString() {
      return "glyph box " + (char) codepoint + " of width " + getWidth();
   }
} // class NormalizedGlyphBox
