package de.uniulm.mathematik.typo.items;

import de.uniulm.mathematik.typo.fonts.*;

/**
 * The NamedGlyphBox class supports boxes for individual glyphs
 * which are known by their name. It works like NamedGlyphBox but its
 * depth and height depend not on the individual metrics of the glyph
 * but that of the chosen font.
 */
public class NormalizedNamedGlyphBox extends Box {
   private FontMetrics fm;
   private String name;
   private int codepoint;
   private int size;
   private int height;
   private int depth;

   /**
    * Construct a glyph box:
    * @param fm The metrics of the selected font.
    * @param size The font size in points (not units!).
    * @param name The name of the glyph.
    */
   public NormalizedNamedGlyphBox(FontMetrics fm, int size, String name,
	 int codepoint) {
      this.fm = fm;
      this.size = size;
      this.name = name;
      this.codepoint = codepoint;
      height = fm.getAscender() * size;
      if (height < 0) {
	 height = 0;
      }
      depth = -fm.getDescender() * size;
      if (depth < 0) {
	 depth = 0;
      }
   }

   /**
    * Returns the width of the glyph in units (1/1000 points).
    * This value is derived from the width returned by the
    * associated font metrics. This is independent from the
    * bounding box.
    */
   public int getWidth() {
      return fm.getWidth(name) * size;
   }

   /**
    * Return the height of the glyph in units (1/1000 points).
    * This value is derived from the ascender of the corresponding
    * font.
    */
   public int getHeight() {
      return height;
   }

   /**
    * Return the depth of the glyph in units (1/1000 points).
    * This value is derived from the descender of the corresponding
    * font.
    */
   public int getDepth() {
      return depth;
   }

   /**
    * Return the character name.
    */
   public String getName() {
      return name;
   }

   /**
    * Return the Unicode codepoint.
    */
   public int getChar() {
      return codepoint;
   }

   /**
    * Return its font
    */
   public FontMetrics getFont() {
      return fm;
   }

   /**
    * Return its font size
    */
   public int getFontSize() {
      return size;
   }

   /**
    * Generate PostScript text that generates the glyph.
    * @param context The context is used to combine multiple
    *                subsequent characters to PostScript strings
    *                as far as possible.
    */
   public StringBuffer genPostScript(PostScriptContext context) {
      return context.addNamedGlyphTo(fm, size, name);
   }

   /**
    * Return a printable string representing the internal state.
    */
   public String toString() {
      return "glyph box '" + name + "' of width " + getWidth();
   }
} // class NormalizedNamedGlyphBox
